/*
    webmail -- simple pop3/smtp web e-mail cgi
    Copyright (C) 2000-2004 Dan Cahill

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef WIN32
#include "autoconf-nt.h"
#else
#include "autoconf.h"
#endif
#include <ctype.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#ifdef WIN32
	#pragma comment(lib, "ws2_32.lib")
	#include <winsock.h>
	#include <io.h>
	#include <sys/timeb.h>
#ifdef BCC
	#define _setmode setmode
#endif
	#define snprintf _snprintf
	#define vsnprintf _vsnprintf
	#define strcasecmp stricmp
	#define strncasecmp strnicmp
	#define sleep(x) Sleep(x*1000)
	#define dlopen(x, y) LoadLibrary(x)
	#define dlsym(x, y) GetProcAddress(x, y)
	#define dlclose(x) FreeLibrary(x)
#else
	#include <dirent.h>
	#include <netdb.h>
	#include <paths.h>
	#include <syslog.h>
	#include <unistd.h>
	#include <netinet/in.h>
	#include <sys/resource.h>
	#include <sys/socket.h>
	#include <sys/time.h>
	#include <sys/wait.h>
	#define closesocket close
#endif
#include "config.h"

typedef struct {
	char From[256];
	char ReplyTo[256];
	char To[256];
 	char Date[128];
	char Subject[256];
	char CC[2048];
	char contenttype[128];
	char boundary[128];
	char encoding[128];
	char status;
} wmheader;
typedef struct {
	char pop3_host[64];
	char smtp_host[64];
	short unsigned int imap_port;
	short unsigned int pop3_port;
	short unsigned int smtp_port;
	char support_imap;
	char support_pop3;
	char use_append_address;
	char append_address[64];
	char usermap[256];
	char allow_userfrom;
	char allow_userhosts;
	char smtp_auth;
	char base_image_url[64];
	char stylesheet[64];
	char colour_bodybg[16];
	char colour_editform[16];
	char colour_ftext[16];
	char colour_links[16];
	char colour_mlisteven[16];
	char colour_mlistodd[16];
	char colour_mlistnew[16];
	char colour_trim[16];
	char colour_trimtext[16];
	char pop_status;
	short unsigned int max_list_size;
	short unsigned int max_runtime;
	unsigned int max_postsize;
	unsigned short int max_recipients;
	char raw_messages;
	char use_frames;
	char use_syslog;
#ifdef HAVE_SQL
	char sql_type[32];
	char sql_host[64];
	char sql_port[10];
	char sql_user[32];
	char sql_pass[32];
	char sql_db[32];
	char sql_dsn[100];
#ifdef HAVE_ADDRESSBOOK
	unsigned short int max_addresses;
#endif
#endif
} config_struct;
typedef struct {
	char ClientIP[16];
	char Connection[128];
	unsigned int ContentLength;
	char ContentType[128];
	char Cookie[1024];
	char Host[128];
	char PathInfo[128];
	char *PostData;
	char QueryString[1024];
	char Referer[128];
	char RequestMethod[8];
	char RequestURI[1024];
	char ScriptName[128];
	char UserAgent[128];
} request_struct;
typedef struct {
	char username[64];
	char password[64];
	char servertype[6];
	char pop3server[64];
	char smtpserver[64];
	char address[128];
	char fullname[128];
	char fromaddress[128];
	char replytoaddress[128];
} user_struct;

#ifdef HAVE_SQL
typedef struct {
	unsigned int ctime;
	unsigned int NumFields;
	unsigned int NumTuples;
	char fields[1024];
	char **cursor;
} SQLREPLY;
#define MAX_TUPLE_SIZE 16384
#define MAX_FIELD_SIZE 12288
#define MAX_QUERIES 2
extern SQLREPLY sqlreply[];
extern int sql_is_connected;
#endif

extern char **environ;
extern config_struct  config;
extern request_struct request;
extern user_struct    user;
extern int connected;
extern int wmsocket;
extern int wmimapidx;
extern int wmimapmread;
extern int wmimapmsize;

/* address.c functions */
void wmaddr_popup(void);
void wmaddr_edit(void);
void wmaddr_list(void);
void wmaddr_save(void);
/* cgi.c functions */
int   IntFromHex(char *pChars);
void  URLDecode(unsigned char *pEncoded);
char *str2html(char *instring);
void  cgi_readenv(void);
void  cgi_readpost(void);
char *getgetenv(char *query);
char *getmimeenv(char *query);
char *getpostenv(char *query);
char *get_mime_type(char *name);
char *strcasestr(const char *src, const char *query);
void  striprn(char *string);
/* codec.c functions */
int   printhex(const char *format, ...);
int   printht(const char *format, ...);
void  printline(short int reply, char *msgtext);
char *decodedate(char *src);
char *DecodeRFC2047(char *src);
int   DecodeHTML(short int reply, char *src, char *ctype, short int crlf);
int   DecodeQP(short int reply, char *src, char *ctype);
int   DecodeText(short int reply, char *src);
int   EncodeBase64(char *src, int srclen);
int   DecodeBase64(char *src, char *ctype);
char *EncodeBase64string(char *src);
char *DecodeBase64string(char *src);
char *EncodeRFC2047(char *src);
int   jis2euc(char *str, char *buf, int buflen);
int   euc2jis(char *str, char *buf, int buflen);
/* config.c functions */
int   config_read(void);
int   config_readuser(void);
/* html.c functions */
void  printheader(void);
void  printfooter(void);
void  printlogin(void);
void  printlogout(void);
void  printframeset(void);
void  printtopframe(void);
void  printleftframe(void);
/* http.c functions */
void  send_header(int cacheable, char *extra_header, char *mime_type, int length, time_t mod);
int   wmcookieget(void);
int   wmcookieset(void);
void  wmcookiekill(void);
/* mime.c functions */
int   webmailheader(wmheader *header);
void  webmailfiledl(void);
char *webmailfileul(char *xfilename, char *xfilesize);
int   webmailmime(char *contenttype, char *encoding, char *boundary, short int nummessage, short int reply, short int depth);
/* server.c functions */
int   wmprintf(const char *format, ...);
int   wmfgets(char *buffer, int max, int fd);
void  wmclose(void);
void  wmexit(void);
int   wmserver_connect(void);
void  wmserver_disconnect(void);
int   wmserver_smtpconnect(void);
int   wmserver_smtpauth(void);
void  wmserver_smtpdisconnect(void);
int   wmserver_count(void);
int   wmserver_msgdele(int message);
int   wmserver_msghead(int message);
int   wmserver_msgretr(int message);
int   wmserver_msgsize(int message);
int   wmserver_uidl(int message, char *uidl);
int   is_msg_end(char *buffer);
/* signals.c functions */
void setsigs(void);
/* sql.c functions */
#ifdef HAVE_SQL
void sql_disconnect();
void sql_freeresult(int sqr);
int  sql_update(char *format, ...);
int  sql_query(char *format, ...);
char *sql_getname(int sqr, int field);
char *sql_getvalue(int sqr, int tuple, int field);
char *sql_getvaluebyname(int sqr, int tuple, char *fieldname);
int  sql_numfields(int sqr);
int  sql_numtuples(int sqr);
char *str2sql(char *inbuffer);
#endif
/* wmmain.c functions */
void  webmaildelete(void);
void  webmaillist(void);
void  webmailraw(void);
void  webmailread(void);
void  webmailsend(void);
void  webmailwrite(void);
