/*
    webmail -- simple pop3/smtp web e-mail cgi
    Copyright (C) 2000-2004 Dan Cahill

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "main.h"

#define RFC1123FMT "%a, %d %b %Y %H:%M:%S GMT"

static short int statussent=0;
static short int sent=0;

void send_header(int cacheable, char *extra_header, char *mime_type, int length, time_t mod)
{
	char timebuf[100];
	time_t t=0;

	if (sent) return;
	/* IIS bursts into flames when we do a redirect, so use nph- */
	if ((strstr(request.ScriptName, "nph-")!=NULL)&&(!statussent)) {
		printf("HTTP/1.0 200 OK\r\n");
	}
	if (length>=0) {
		printf("Content-Length: %d\r\n", length);
	}
	if (mod!=(time_t)-1) {
		strftime(timebuf, sizeof(timebuf), RFC1123FMT, gmtime(&mod));
		printf("Last-Modified: %s\r\n", timebuf);
	}
	if (cacheable) {
		printf("Cache-Control: public\r\n");
		printf("Pragma: public\r\n");
	} else {
		strftime(timebuf, sizeof(timebuf), RFC1123FMT, gmtime(&t));
		printf("Expires: %s\r\n", timebuf);
		printf("Cache-Control: no-cache; must-revalidate\r\n");
		printf("Pragma: no-cache\r\n");
	}
	if (extra_header!=(char*)0) {
		printf("Content-Type: %s\r\n", mime_type);
	} else {
		printf("Content-Type: text/html\r\n");
	}
	printf("\r\n");
	sent=1;
	return;
}

/* see http://www.netscape.com/newsref/std/cookie_spec.html for cookie handling stuff */
int wmcookieget()
{
	char timebuffer[100];
	char temppasswd[100];
	char *ptemp;
	time_t t;

	memset(temppasswd, 0, sizeof(temppasswd));
	memset(user.username, 0, sizeof(user.username));
	memset(user.password, 0, sizeof(user.password));
	memset(user.servertype, 0, sizeof(user.servertype));
	memset(user.pop3server, 0, sizeof(user.pop3server));
	memset(user.smtpserver, 0, sizeof(user.smtpserver));
	ptemp=strcasestr(request.Cookie, "wminfo=");
	if (ptemp==NULL) return -1;
	ptemp+=7;
	while ((*ptemp)&&(*ptemp!=':')&&(*ptemp!=';')&&(strlen(user.username)<sizeof(user.username)))
		user.username[strlen(user.username)]=*ptemp++;
	ptemp++;
	while ((*ptemp)&&(*ptemp!=':')&&(*ptemp!=';')&&(strlen(user.servertype)<sizeof(user.servertype)-1))
		user.servertype[strlen(user.servertype)]=*ptemp++;
	if (config.support_imap!='y') {
		snprintf(user.servertype, sizeof(user.servertype)-1, "POP3");
	}
	if (config.support_pop3!='y') {
		snprintf(user.servertype, sizeof(user.servertype)-1, "IMAP");
	}
	if (config.allow_userhosts=='y') {
		ptemp++;
		while ((*ptemp)&&(*ptemp!=':')&&(*ptemp!=';')&&(strlen(user.pop3server)<sizeof(user.pop3server)-1))
			user.pop3server[strlen(user.pop3server)]=*ptemp++;
		ptemp++;
		while ((*ptemp)&&(*ptemp!=':')&&(*ptemp!=';')&&(strlen(user.smtpserver)<sizeof(user.smtpserver)-1))
			user.smtpserver[strlen(user.smtpserver)]=*ptemp++;
	} else {
		snprintf(user.pop3server, sizeof(user.pop3server)-1, "%s", config.pop3_host);
		snprintf(user.smtpserver, sizeof(user.smtpserver)-1, "%s", config.smtp_host);
	}
	ptemp=strcasestr(request.Cookie, "wmpass=");
	if (ptemp==NULL) return -1;
	ptemp+=7;
	while ((*ptemp)&&(*ptemp!=';')&&(strlen(temppasswd)<sizeof(temppasswd)))
		temppasswd[strlen(temppasswd)]=*ptemp++;
	snprintf(user.password, sizeof(user.password), "%s", DecodeBase64string(temppasswd));
	t=time((time_t*)0)+604800;
	strftime(timebuffer, sizeof(timebuffer), RFC1123FMT, gmtime(&t));
	if (strlen(user.password)==0) return -1;
	/* IIS bursts into flames when we do a redirect, so use nph- */
	if ((strstr(request.ScriptName, "nph-")!=NULL)&&(!statussent)) {
		printf("HTTP/1.0 200 OK\r\n");
		statussent=1;
	}
	printf("Set-Cookie: wminfo=%s:%s:%s:%s; path=%s/; expires=%s\r\n", user.username, user.servertype, user.pop3server, user.smtpserver, request.ScriptName, timebuffer);
	printf("Set-Cookie: wmpass=%s; path=%s/\r\n", temppasswd, request.ScriptName);
	if (strlen(user.password)==0) return -1;
	return 0;
}

int wmcookieset()
{
	time_t t;
	char timebuffer[100];

	if (strcasecmp(request.RequestMethod, "POST")!=0) return -1;
	if (getpostenv("WMUSERNAME")!=NULL) {
		strncpy(user.username, getpostenv("WMUSERNAME"), sizeof(user.username)-1);
	}
	if (getpostenv("WMPASSWORD")!=NULL) {
		strncpy(user.password, getpostenv("WMPASSWORD"), sizeof(user.password)-1);
	}
	if (getpostenv("WMSERVERTYPE")!=NULL) {
		strncpy(user.servertype, getpostenv("WMSERVERTYPE"), sizeof(user.servertype)-1);
	}
	if (config.support_imap!='y') {
		snprintf(user.servertype, sizeof(user.servertype)-1, "POP3");
	}
	if (config.support_pop3!='y') {
		snprintf(user.servertype, sizeof(user.servertype)-1, "IMAP");
	}
	if (config.allow_userhosts=='y') {
		if (getpostenv("WMPOP3SERVER")!=NULL) {
			strncpy(user.pop3server, getpostenv("WMPOP3SERVER"), sizeof(user.pop3server)-1);
		}
		if (getpostenv("WMSMTPSERVER")!=NULL) {
			strncpy(user.smtpserver, getpostenv("WMSMTPSERVER"), sizeof(user.smtpserver)-1);
		}
	} else {
		snprintf(user.pop3server, sizeof(user.pop3server)-1, "%s", config.pop3_host);
		snprintf(user.smtpserver, sizeof(user.smtpserver)-1, "%s", config.smtp_host);
	}
	/* thanks to jberkes@pc-tools.net for the syslog addition */
	if (wmserver_connect()==0) {
		t=time((time_t*)0)+604800;
		strftime(timebuffer, sizeof(timebuffer), RFC1123FMT, gmtime(&t));
		printf("Set-Cookie: wminfo=%s:%s:%s:%s; path=%s/; expires=%s\r\n", user.username, user.servertype, user.pop3server, user.smtpserver, request.ScriptName, timebuffer);
		printf("Set-Cookie: wmpass=%s; path=%s/\r\n", EncodeBase64string(user.password), request.ScriptName);
		printf("Cache-Control: no-cache; must-revalidate\r\n");
		printf("Pragma: no-cache\r\n");
		printf("Content-Type: text/html\r\n");
		printf("\r\n");
		sent=1;
		if (strstr(request.UserAgent, "Lynx")==NULL) {
			printf("<HTML><BODY>\r\n");
			printf("<CENTER>The requested resource can be found at <A HREF=%s/>%s/</A>.</CENTER>\r\n", request.ScriptName, request.ScriptName);
			printf("<SCRIPT LANGUAGE=JavaScript>\r\n<!--\r\nlocation.replace('%s/');\r\n// -->\r\n</SCRIPT>\r\n", request.ScriptName);
			printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"1; URL='%s/'\">\r\n", request.ScriptName);
			printf("</BODY></HTML>\r\n");
		}
#ifndef WIN32
		if (config.use_syslog=='y') {
			syslog(LOG_MAIL|LOG_INFO, "%s - %s @ %s success", request.ClientIP, user.username, user.pop3server);
		}
#endif
		return 0;
	}
#ifndef WIN32
	if (config.use_syslog=='y') {
		syslog(LOG_MAIL|LOG_INFO, "%s - Invalid username or password for %s @ %s", request.ClientIP, user.username, user.pop3server);
	}
#endif
	wmexit();
	return -1;
}

void wmcookiekill()
{
	/* IIS bursts into flames when we do a redirect, so use nph- */
	if ((strstr(request.ScriptName, "nph-")!=NULL)&&(!statussent)) {
		printf("HTTP/1.0 200 OK\r\n");
		statussent=1;
	}
	printf("Set-Cookie: wmpass=; path=%s/\r\n", request.ScriptName);
	printf("Cache-Control: no-cache; must-revalidate\r\n");
	printf("Pragma: no-cache\r\n");
	printf("Content-Type: text/html\r\n");
	printf("\r\n");
	sent=1;
	printf("<HTML><BODY>\r\n");
	printf("<CENTER>The requested resource can be found at <A HREF=%s/>%s/</A>.</CENTER>\r\n", request.ScriptName, request.ScriptName);
	printf("<SCRIPT LANGUAGE=JavaScript>\r\n<!--\r\nlocation.replace('%s/');\r\n// -->\r\n</SCRIPT>\r\n", request.ScriptName);
	printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"1; URL='%s/'\">\r\n", request.ScriptName);
	printf("</BODY></HTML>\r\n");
	return;
}
