/*
    webmail -- simple pop3/smtp web e-mail cgi
    Copyright (C) 2000-2004 Dan Cahill

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "main.h"
static struct sockaddr_in server;
static struct hostent *hp;
#ifdef WIN32
static WSADATA wsaData;
#endif
static char inbuffer[1024];
static char *ptemp;

/*
void logdata(const char *format, ...)
{
	FILE *fp;
	va_list ap;
	char buffer[1024];
#ifdef WIN32
	char *file="C:\\webmail.log";
#else
	char *file="/tmp/webmail.log";
#endif

	fp=fopen(file, "a");
	if (fp!=NULL) {
		memset(buffer, 0, sizeof(buffer));
		va_start(ap, format);
		vsnprintf(buffer, sizeof(buffer)-1, format, ap);
		va_end(ap);
		striprn(buffer);
		fprintf(fp, "%s\r\n", buffer);
		fclose(fp);
	}
}
*/

int wmprintf(const char *format, ...)
{
	char buffer[1024];
	va_list ap;
	int len;

	memset(buffer, 0, sizeof(buffer));
	va_start(ap, format);
	vsnprintf(buffer, sizeof(buffer)-1, format, ap);
	va_end(ap);
	send(wmsocket, buffer, strlen(buffer), 0);
	len=strlen(buffer);
//	striprn(buffer);
//	logdata (">> %s", buffer);
	return len;
}

int wmfgets(char *buffer, int max, int fd)
{
	char *pbuffer=buffer;
	char temp[2];
	int n=0;

	memset(temp, 0, sizeof(temp));
	while (n<max) {
		if (recv(fd, temp, 1, 0)<1) {
			connected=0;
			wmexit();
		}
		wmimapmread++;
		n++;
		if (strncasecmp(user.servertype, "pop3", 4)==0) {
			if ((temp[0]=='.')&&(pbuffer[0]=='.')&&(n==2)) continue;
		}
		if (temp[0]==13) continue;
		*buffer=temp[0];
		buffer++;
		if (temp[0]==10) break;
	}
	*buffer=0;
	striprn(pbuffer);
//	logdata ("<< %s\r\n", pbuffer);
	return n;
}

void wmclose()
{
	char junk[10];

	shutdown(wmsocket, 2);
	while (recv(wmsocket, junk, sizeof(junk), 0)>0);
	closesocket(wmsocket);
	fflush(stdout);
	return;
}

void wmexit()
{
	wmserver_disconnect();
	wmclose();
#ifdef WIN32
	WSACleanup();
#endif
	exit(0);
}

int wmserver_smtpconnect()
{
	/* some smtp servers like pop auth before smtp */
	if (wmserver_connect()!=0) return -1;
	wmserver_disconnect();
	if ((hp=gethostbyname(user.smtpserver))==NULL) {
		printf("<CENTER>%s '%s'</CENTER>\n", ERR_DNS_SMTP, user.smtpserver);
		return -1;
	}
	memset((char *)&server, 0, sizeof(server));
	memmove((char *)&server.sin_addr, hp->h_addr, hp->h_length);
	server.sin_family=hp->h_addrtype;
	server.sin_port=(unsigned short)htons(config.smtp_port);
	if ((wmsocket=socket(AF_INET, SOCK_STREAM, 0))<0) return -1;
//	setsockopt(wmsocket, SOL_SOCKET, SO_KEEPALIVE, 0, 0);
	if (connect(wmsocket, (struct sockaddr *)&server, sizeof(server))<0) {
		printf("<CENTER>%s '%s'</CENTER>\n", ERR_CON_SMTP, user.smtpserver);
		return -1;
	}
	do {
		memset(inbuffer, 0, sizeof(inbuffer));
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
	} while ((inbuffer[3]!=' ')&&(inbuffer[3]!='\0'));
	return 0;
}

int wmserver_smtpauth()
{
	char authstring[200];

	/* Thanks to Ryan Brown <ryan@wabdo.com> for the improved SMTP AUTH patch. */
	wmprintf("AUTH PLAIN ");
	snprintf(authstring, sizeof(authstring)-1, "%s%c%s%c%s", user.username, '\0', user.username, '\0', user.password);
	EncodeBase64(authstring, (strlen(user.username)*2)+strlen(user.password)+2);
	do {
		memset(inbuffer, 0, sizeof(inbuffer));
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
	} while ((inbuffer[3]!=' ')&&(inbuffer[3]!='\0'));
	if (strncasecmp(inbuffer, "502", 3)==0) return 0;
	if (strncasecmp(inbuffer, "235", 3)!=0) return -1;
	return 0;
}

void wmserver_smtpdisconnect()
{
	wmprintf("QUIT\r\n");
	do {
		memset(inbuffer, 0, sizeof(inbuffer));
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
	} while ((inbuffer[3]!=' ')&&(inbuffer[3]!='\0'));
	wmclose();
}

int wmserver_connect()
{
	int retries;

	if (connected) return 0;
	if ((strlen(user.username)==0)||(strlen(user.password)==0)) {
		send_header(0, "1", "text/html", -1, -1);
		printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
		printf("<CENTER>%s</CENTER><BR>\n", ERR_NOUSER);
		wmexit();
	}
	if ((strlen(user.pop3server)==0)||(strlen(user.smtpserver)==0)) {
		send_header(0, "1", "text/html", -1, -1);
		printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
		printf("<CENTER>%s</CENTER><BR>\n", ERR_NOHOST);
		wmexit();
	}
#ifdef WIN32
	if (WSAStartup(0x101, &wsaData)) {
		send_header(0, "1", "text/html", -1, -1);
		printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
		printf("<CENTER>%s</CENTER><BR>\n", ERR_NOWINSOCK);
		wmexit();
	}
#endif
	if ((hp=gethostbyname(user.pop3server))==NULL) {
		send_header(0, "1", "text/html", -1, -1);
		printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
		printf("<CENTER>%s '%s'</CENTER><BR>\n", ERR_DNS_POP3, user.pop3server);
		wmexit();
	}
	for (retries=5;retries>0;retries--) {
		memset((char *)&server, 0, sizeof(server));
		memmove((char *)&server.sin_addr, hp->h_addr, hp->h_length);
		server.sin_family=hp->h_addrtype;
		if (strncasecmp(user.servertype, "pop3", 4)==0) {
			server.sin_port=htons(config.pop3_port);
		} else if (strncasecmp(user.servertype, "imap", 4)==0) {
			server.sin_port=htons(config.imap_port);
		}
		if ((wmsocket=socket(AF_INET, SOCK_STREAM, 0))<0) return -1;
		if (connect(wmsocket, (struct sockaddr *)&server, sizeof(server))<0) {
			send_header(0, "1", "text/html", -1, -1);
			printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
			printf("<CENTER>%s '%s'</CENTER><BR>\n", ERR_CON_POP3, user.pop3server);
			wmexit();
		}
		connected=1;
		/* Check current status */
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
			/* Send user name */
			wmprintf("USER %s\r\n", user.username);
			wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			if (strncasecmp(inbuffer, "+OK", 3)!=0) {
				ptemp=strchr(inbuffer, ' ');
				send_header(0, "1", "text/html", -1, -1);
				printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
				if (ptemp) printf("<CENTER>%s</CENTER><BR>\n", ptemp);
				return -1;
			}
			/* Send password */
			wmprintf("PASS %s\r\n", user.password);
			wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			if (strncasecmp(inbuffer, "+OK", 3)!=0) {
				ptemp=strchr(inbuffer, ' ');
				if ((strcasestr(ptemp, "pop lock busy")!=NULL)&&(retries>1)) {
					wmserver_disconnect();
					sleep(1);
					continue;
				}
				send_header(0, "1", "text/html", -1, -1);
				printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
				if (ptemp) printf("<CENTER>%s</CENTER><BR>\n", ptemp);
				return -1;
			}
			return 0;
		} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
			/* Send user name */
			wmimapidx++;
			wmprintf("%d LOGIN %s %s\r\n", wmimapidx, user.username, user.password);
			for (;;) {
				wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
				if (isdigit(inbuffer[0])) break;
			}
			ptemp=inbuffer;
			while ((ptemp)&&(*ptemp!=' ')) ptemp++;
			if (*ptemp) ptemp++;
			if (strncasecmp(ptemp, "OK", 2)!=0) {
				ptemp=strchr(inbuffer, ' ');
				send_header(0, "1", "text/html", -1, -1);
				printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"2; URL=%s/\">\n", request.ScriptName);
				if (ptemp) printf("<CENTER>%s</CENTER><BR>\n", ptemp);
				wmexit();
			}
//			wmimapidx++;
//			wmprintf("%d LIST \"\" \"*\"\r\n", wmimapidx);
//			for (;;) {
//				wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
//				if (isdigit(inbuffer[0])) break;
//			}
			wmimapidx++;
			wmprintf("%d SELECT INBOX\r\n", wmimapidx);
			for (;;) {
				wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
				if (isdigit(inbuffer[0])) break;
			}
			return 0;
		}
		return -1;
	}
	return -1;
}

void wmserver_disconnect()
{
	if (!connected) return;
	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("QUIT\r\n");
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		wmclose();
		connected=0;
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d LOGOUT\r\n", wmimapidx);
		for (;;) {
			wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			if (isdigit(inbuffer[0])) break;
		}
		wmclose();
		connected=0;
	}
	return;
}

int wmserver_count()
{
	int count=-1;

	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("STAT\r\n");
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if (strncasecmp(inbuffer, "+OK", 3)!=0) return -1;
		ptemp=inbuffer;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		ptemp++;
		return atoi(ptemp);
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d STATUS INBOX (MESSAGES)\r\n", wmimapidx);
		for (;;) {
			wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			if (isdigit(inbuffer[0])) break;
			if (strncasecmp(inbuffer, "* STATUS", 8)==0) {
				ptemp=inbuffer;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				count=atoi(ptemp);
			}
		}
		ptemp=inbuffer;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		if (*ptemp) ptemp++;
		if (strncasecmp(ptemp, "OK", 2)!=0) return -1;
		return count;
	}
	return -1;
}

int wmserver_msgdele(int message)
{
	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("DELE %d\r\n", message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if (strncasecmp(inbuffer, "+OK", 3)!=0) return -1;
		return 0;
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d STORE %d +FLAGS (\\Deleted)\r\n", wmimapidx, message);
		for (;;) {
			wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			if (isdigit(inbuffer[0])) break;
		}
		wmimapidx++;
		wmprintf("%d EXPUNGE\r\n", wmimapidx);
		for (;;) {
			wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			if (isdigit(inbuffer[0])) break;
		}
		ptemp=inbuffer;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		if (*ptemp) ptemp++;
		if (strncasecmp(ptemp, "OK", 2)!=0) {
			ptemp=strchr(inbuffer, ' ');
			if (ptemp) printf("<CENTER>%s</CENTER>\n", ptemp);
			wmclose();
			return -1;
		}
		return 0;
	}
	return -1;
}

int wmserver_msghead(int message)
{
	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("TOP %d 0\r\n", message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if (strncasecmp(inbuffer, "+OK", 3)!=0) return -1;
		return 0;
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d FETCH %d BODY[HEADER]\r\n", wmimapidx, message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		return 0;
	}
	return -1;
}

int wmserver_msgretr(int message)
{
	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("RETR %d\r\n", message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if (strncasecmp(inbuffer, "+OK", 3)!=0) return -1;
		return 0;
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d FETCH %d RFC822\r\n", wmimapidx, message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		ptemp=strchr(inbuffer, '{');
		if (*ptemp) ptemp++;
		if (*ptemp) {
			wmimapmsize=atoi(ptemp);
		} else {
			wmimapmsize=-1;
		}
		wmimapmread=0;
		return 0;
	}
	return -1;
}

int wmserver_msgsize(int message)
{
	int size=-1;

	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("LIST %d\r\n", message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if (strncasecmp(inbuffer, "+OK", 3)!=0) return -1;
		ptemp=inbuffer;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		ptemp++;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		ptemp++;
		return atoi(ptemp);
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d FETCH %d RFC822.SIZE\r\n", wmimapidx, message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		for (;;) {
			if (isdigit(inbuffer[0])) break;
			if (strcasestr(inbuffer, "FETCH")!=NULL) {
				ptemp=inbuffer;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				while ((ptemp)&&(*ptemp!=' ')) ptemp++;
				if (*ptemp) ptemp++;
				size=atoi(ptemp);
			}
		}
		return size;
	}
	return -1;
}

int wmserver_uidl(int message, char *uidl)
{
	static const char Base64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.";
	char uidltemp[71];
	unsigned char a, b, c, d;
	int dst, i, enclen, remlen;

	memset(uidltemp, 0, sizeof(uidltemp));
	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		wmprintf("UIDL %d\r\n", message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		if (strncasecmp(inbuffer, "+OK", 3)!=0) return -1;
		ptemp=inbuffer;
		while ((*ptemp)&&(*ptemp!=' ')) ptemp++;
		ptemp++;
		while ((*ptemp)&&(*ptemp!=' ')) ptemp++;
		ptemp++;
		snprintf(uidltemp, 70, "%s", ptemp);
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		wmimapidx++;
		wmprintf("%d FETCH %d UID\r\n", wmimapidx, message);
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
		ptemp=inbuffer;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		if (*ptemp) ptemp++;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		if (*ptemp) ptemp++;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		if (*ptemp) ptemp++;
		while ((ptemp)&&(*ptemp!=' ')) ptemp++;
		if (*ptemp) ptemp++;
		snprintf(uidltemp, 70, "%s", ptemp);
		ptemp=uidltemp;
		while ((ptemp)&&(*ptemp!=')')) ptemp++;
		if (*ptemp) *ptemp='\0';
		wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
	}
	ptemp=uidltemp;
	dst=0;
	enclen=strlen(uidltemp)/3;
	remlen=strlen(uidltemp)-3*enclen;
	for (i=0;i<enclen;i++) {
		a=(ptemp[0]>>2);
	        b=(ptemp[0]<<4)&0x30;
		b|=(ptemp[1]>>4);
		c=(ptemp[1]<<2)&0x3c;
		c|=(ptemp[2]>>6);
		d=ptemp[2]&0x3f;
		ptemp+=3;
		uidl[dst++]=Base64[a];
		uidl[dst++]=Base64[b];
		uidl[dst++]=Base64[c];
		uidl[dst++]=Base64[d];
	}
	if (remlen==1) {
		a=(ptemp[0]>>2);
		b=(ptemp[0]<<4)&0x30;
		uidl[dst++]=Base64[a];
		uidl[dst]=Base64[b];
	} else if (remlen==2) {
		a=(ptemp[0]>>2);
		b=(ptemp[0]<<4)&0x30;
		b|=(ptemp[1]>>4);
		c=(ptemp[1]<<2)&0x3c;
		uidl[dst++]=Base64[a];
		uidl[dst++]=Base64[b];
		uidl[dst]=Base64[c];
	}
	return 0;
}

int is_msg_end(char *buffer)
{
	if ((config.support_pop3=='y')&&(strncasecmp(user.servertype, "pop3", 4)==0)) {
		if (strcmp(buffer, ".")==0) return 1;
	} else if ((config.support_imap=='y')&&(strncasecmp(user.servertype, "imap", 4)==0)) {
		if ((wmimapmsize>0)&&(wmimapmread<wmimapmsize)) return 0;
		if (strcmp(buffer, ")")==0) {
			for (;;) {
				if (isdigit(inbuffer[0])) {
					if (atoi(inbuffer)==wmimapidx) break;
				}
				wmfgets(inbuffer, sizeof(inbuffer)-1, wmsocket);
			}
			return 1;
		}
	}
	return 0;
}
